/*____________________________________________________________________________
 *
 * vcif - program for validating a Crystallographic Information File
 *
 * Copyright (c) 1995-2004 International Union of Crystallography
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *
 * Author: Brian McMahon, IUCr, 5 Abbey Square, Chester CH1 2HU, England
 * E-mail: bm@iucr.org
 *
 * Rev 1.2:  H. J. Bernstein, Dowling College, Oakdale, NY 11769, USA
 * E-mail: yaya@dowling.edu

____________________________________________________________________________*/

/*
The Crystallographic Information File (CIF) has been developed by the IUCr
as a standard for the electronic archival and transmission of
crystallographic data.  A CIF may contain numeric, character or textual
data, and is thus suited to the purpose of transmitting the text of a
paper reporting a crystallographic structure determination to IUCr
journals, primarily Acta Crystallographica, Section C. This program checks
the syntax of a file against the standard syntax rules for STAR files and
against the more restrictive CIF rules.

The Crystallographic Information File is described in the paper "The
Crystallographic Information File (CIF): a New Standard Archive File for
Crystallography" by S. R. Hall, F. H. Allen & I. D. Brown [Acta
Cryst. (1991), A45, 655-685], copies of which may be obtained from the
Technical Editor, IUCr, 5 Abbey Square, Chester CH1 2HU, England.

The STAR file is described by S. R. Hall
[J. Chem. Inf. Comput. Sci. (1991), 31, 326-333] and by S. R. Hall &
N. Spadaccini [J. Chem. Inf. Comput. Sci.  (1994), 34, 505-508].

The STAR concept is protected by UK Patent No. GB 2 243 467 B.

Copyright in the CIF specifications and Dictionary resides with the IUCr.
*/

#define VERSION "1.2"	/* Program version number */

/* Version history

1.2  2005-01-31 Start of changes for CIF 1.1:  allow long
                lines up to 2048, allow 75 character data names
				revise argument list processing, allowing
				long arguments names with -- prefix, adding
				"--ciflevel 0" argument to restore CIF 1.0
				to restore old processing.

1.1  2004-12-07 Release of vcif under GPL

0.58 2003-04-24 Return at most 255 as no. of errors_reported (because
                shell evaluates $? modulo 256)

0.57 2001-10-12 Minor change to verbose error message for loops

0.56 1998-09-17 Added option -b to print out data block names only

0.55 1998-09-03 Some movement of declarations and explicit casts to reduce
                compiler warnings and errors - no significant changes

0.54 1998-08-26 The line
                sprintf(message, "ERROR: Duplicate dataname %s in data_%s at \
                line %d\n", bitof(dname), bitof(current_dblock), lineno);
                in pass1.c was changed to
                sprintf(message, "ERROR: Duplicate dataname %s in data_%s at \
                line %d\n", dname, bitof(current_dblock), lineno);
                (i.e. only one occurrence of bitof()) because otherwise
                the result was "Duplicate dataname X in data_X..."
0.53 1998-08-06 A misplaced case statement in get_type_of(token) led to
                strings of the type _dataname' within a text field being
                interpreted as of type ENDSQUOT

                Error/warning messages for datanames > 32 characters
                suppressed (CIF Core 2.0 lifts this restriction): new
                value of 74 parametrised in pass1.h to take account of the
                retained 80-character linelength restriction and the need for
                datablock names including datanames in dictionaries

                Expanded usage() and introduced -V, -h options

0.52 1998-07-05 Subroutine bitof() introduced to restrict length of long
                tokens passed to ERROR messages (via fixed-length
                message[256] buffer). Uses magic numbers for arbitrary
                cut-off length - this should be parametrised at some point

0.51 1998-06-19 Printed out the first portion of the contents of the offending
                line in verbose mode (for users with difficulty locating line
                numbers)
*/

#define NO 0		/* no */
#define YES 1		/* yes */

#define CIFMAXDNAME 75	/* maximum length of CIF data name - originally 32,
			               now 74 (because maximum line length 80 in CIF 1.0)
						   or 75 (CIF 1.1) */

/* state codes */
#define BEGINNING_OF_FILE 0
#define IN_LOOP_EXPECTING_DNAME 1
#define IN_LOOP_EXPECTING_VALUE 2
#define IN_LOOP_BUILDING_SQUOT 3
#define IN_LOOP_BUILDING_DQUOT 4
#define IN_LOOP_BUILDING_TEXT 5
#define EXPECTING_DNAME 6
#define EXPECTING_VALUE 7
#define BUILDING_SQUOT 8
#define BUILDING_DQUOT 9
#define BUILDING_TEXT 10

/* token types */
#define BLOCKCODE 0
#define DATANAME 1
#define CHAR 2
#define COMMENT 3
#define BEGTEXT 4
#define ENDTEXT 5
#define LOOPCODE 6
#define BEGSQUOT 7
#define BEGDQUOT 8
#define ENDSQUOT 9
#define ENDDQUOT 10
#define GLOBAL 11
#define SAVE_BEGIN 12
#define SAVE_END 13
#define UNDEFINED 99

/* function prototypes */

int elaborate(int);
void load_dictionary_info( void );
int usage( void );
void vcifwarn(char *, int);
void pass2( void );

void pass1(FILE *);
int get_type_of(char *);
void check_dataname(char *);
void check_dblkname(char *);
void check_savename(char *);
unsigned int hash(char *);
struct nlist *lookup(char *);
struct nlist *install(char *);
char *bitof(char *);
